/*----------------------------------------------------------------------
  File    : TSP.java
  Contents: a traveling salesman problem
  Author  : Christian Borgelt
  History : 2005.11.19 file created
            2016.04.07 StringBuffer replaced by StringBuilder
----------------------------------------------------------------------*/
package acopt;

import java.io.IOException;
import java.io.InputStream;
import java.io.File;
import java.io.Serializable;
import java.io.Reader;
import java.util.Random;
import util.Scanner;

/*--------------------------------------------------------------------*/
/** Class for a traveling salesman problem (TSP).
 *  @since  2005.11.19 (Christian Borgelt) */
/*--------------------------------------------------------------------*/
public class TSP implements Serializable {

  private static final long serialVersionUID = 0x00010000L;

  /* --- constants --- */
  /** block size for array enlargement */
  private static final int BLKSIZE = 16;

  /* --- instance variables --- */
  /** current number of vertices */
  protected int        size;
  /** x-coordinates of vertices */
  protected double[]   xs;
  /** y-coordinates of vertices */
  protected double[]   ys;
  /** distances between vertices (distance matrix) */
  protected double[][] dists;
  /** flag for symmetric distances */
  protected boolean    sym;
  /** flag for euclidean distances */
  protected boolean    euclid;
  /** best (known) tour as an array of vertex indices */
  protected int[]      tour;
  /** bounding box of vertices: x-coordinate of lower left corner */
  private   double     bbx;
  /** bounding box of vertices: y-coordinate of lower left corner */
  private   double     bby;
  /** bounding box of vertices: width */
  private   double     bbw;
  /** bounding box of vertices: height */
  private   double     bbh;
  /** flag for valid bounding box */
  private   boolean    valid;

  /*------------------------------------------------------------------*/
  /** Create a traveling salesman problem (TSP).
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP ()
  { this(BLKSIZE); }

  /*------------------------------------------------------------------*/
  /** Create a traveling salesman problem (TSP).
   *  @param  size  the number of vertices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (int size)
  {                             /* --- create a traveling salesman p. */
    this.size   = 0;            /* initialize the variables */
    this.xs     = new double[size];
    this.ys     = new double[size];
    this.dists  = null;         /* do not create distance matrix yet */
    this.euclid = true;         /* default: Euclidean distances */
    this.sym    = true;         /* default: symmetric distances */
    this.tour   = null;         /* there is no known (best) tour */
    this.valid  = false;        /* the bounding box is not valid */
  }  /* TSP() */

  /*------------------------------------------------------------------*/
  /** Create a traveling salesman problem (TSP).
   *  @param  size  the number of vertices
   *  @param  rand  the random number generator
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (int size, Random rand)
  {                             /* --- create a traveling salesman p. */
    this(size);                 /* initialize the variables, */
    this.randomize(rand);       /* create random vertices, and */
    this.makeDists(true);       /* calculate the distances */
  }  /* TSP() */

  /*------------------------------------------------------------------*/
  /** Resize a traveling salesman problem (TSP).
   *  @param  size  the new number of vertices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  private void resize (int size)
  {                             /* --- resize coord. vectors */
    int      k;                 /* current/new vector size */
    double[] v;                 /* buffer for reallocation */

    k = this.size;              /* get the number of nodes to copy */
    if (size < 0) size = k +((k < BLKSIZE) ? BLKSIZE : (k >> 1));
    if (size < k) k = this.size = size;
    System.arraycopy(this.xs, 0, v = new double[size], 0, k);
    this.xs = v;                /* enlarge the x-coordinates vector */
    System.arraycopy(this.ys, 0, v = new double[size], 0, k);
    this.ys = v;                /* enlarge the y-coordinates vector */
  }  /* resize() */

  /*------------------------------------------------------------------*/
  /** Add a vertex to a traveling salesman problem.
   *  @param  x  the x-coordinate of the vertex to add
   *  @param  y  the y-coordinate of the vertex to add
   *  @return the index of the new vertex
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int add (double x, double y)
  {                             /* --- add a vertex */
    if (this.size >= this.xs.length)
      this.resize(-1);          /* resize the coord. vectors if nec. */
    this.valid = false;         /* bounding box is no longer valid */
    this.xs[this.size] = x;     /* set the coordinates */
    this.ys[this.size] = y;     /* of the new vertex */
    return this.size++;         /* and return its index */
  }  /* add() */

  /*------------------------------------------------------------------*/
  /** Create random vertices for a traveling salesman problem (TSP).
   *  @param  rand  the random number generator
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void randomize (Random rand)
  {                             /* --- create random vertices */
    for (int i = this.size = this.xs.length; --i >= 0; ) {
      this.xs[i] = rand.nextDouble();
      this.ys[i] = rand.nextDouble();
    }                           /* set random coordinates */
    this.valid = false;         /* bounding box is no longer valid */
  }  /* randomize() */

  /*------------------------------------------------------------------*/
  /** Get the number of vertices of the traveling salesman problem.
   *  @return the number of vertices of the traveling salesman problem
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int size ()
  { return this.size; }

  /*------------------------------------------------------------------*/
  /** Get the x-coordinate of a vertex.
   *  @param  i  the index of the vertex
   *  @return the x-coordinate of the vertex
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getX (int i)
  { return this.xs[i]; }

  /*------------------------------------------------------------------*/
  /** Get the y-coordinate of a vertex.
   *  @param  i  the index of the vertex
   *  @return the y-coordinate of the vertex
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getY (int i)
  { return this.ys[i]; }

  /*------------------------------------------------------------------*/
  /** Set the position of a vertex.
   *  @param  i  the index of the vertex
   *  @param  x  the x-coordinate of the vertex
   *  @param  y  the y-coordinate of the vertex
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void setPos (int i, double x, double y)
  { this.xs[i] = x; this.ys[i] = y; }

  /*------------------------------------------------------------------*/
  /** Transform the vertex coordinates.
   *  @param  scale  the scaling factor
   *  @param  xoff   the offset in x-direction
   *  @param  yoff   the offset in y-direction
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void transform (double scale, double xoff, double yoff)
  {                             /* --- transform vertex coordinates */
    for (int i = this.size; --i >= 0; ) {
      this.xs[i] = this.xs[i] *scale +xoff;
      this.ys[i] = this.ys[i] *scale +yoff;
    }                           /* traverse and transform vertices */
    this.valid = false;         /* bounding box is no longer valid */
  }  /* transform() */

  /*------------------------------------------------------------------*/
  /** Compute the counding box of the vertices.
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  private void bbox ()
  {                             /* --- compute bounding box */
    int    i;                   /* loop variable */
    double x, y;                /* coordinates of a vertex */
    double xmax, ymax;          /* maximal x- and y-coordinates */

    this.bbx = Double.MAX_VALUE; xmax = -Double.MAX_VALUE;
    this.bby = Double.MAX_VALUE; ymax = -Double.MAX_VALUE;
    for (i = this.xs.length; --i >= 0; ) {
      x = this.xs[i];          /* traverse the vertices */
      y = this.ys[i];          /* of the problem */
      if (x < this.bbx) this.bbx = x;
      if (x > xmax)     xmax     = x;
      if (y < this.bby) this.bby = y;
      if (y > ymax)     ymax     = y;
    }                           /* find minimum and maximum coords. */
    this.bbw = xmax -this.bbx;  /* compute the width and height */
    this.bbh = ymax -this.bby;  /* of the bounding box */
    this.valid = true;          /* the bounding box is now valid */
  }  /* bbox() */

  /*------------------------------------------------------------------*/
  /** Get the x-coordinate of the lower left corner of the bounding box.
   *  @return the x-coordinate of the lower left corner
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getX ()
  { if (!this.valid) this.bbox(); return this.bbx; }

  /*------------------------------------------------------------------*/
  /** Get the y-coordinate of the lower left corner of the bounding box.
   *  @return the y-coordinate of the lower left corner
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getY ()
  { if (!this.valid) this.bbox(); return this.bby; }

  /*------------------------------------------------------------------*/
  /** Get the width of the bounding box.
   *  @return the width of the bounding box
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getWidth ()
  { if (!this.valid) this.bbox(); return this.bbw; }

  /*------------------------------------------------------------------*/
  /** Get the height of the bounding box.
   *  @return the height of the bounding box
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getHeight ()
  { if (!this.valid) this.bbox(); return this.bbh; }

  /*------------------------------------------------------------------*/
  /** Create and calculate the distance matrix.
   *  @param  calc  whether to calculate the distances
   *                (default: only create the matrix)
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void makeDists (boolean calc)
  {                             /* --- calculate distance matrix */
    int      i, k;              /* loop variables */
    double   dx, dy;            /* coordinate-wise distances */
    double[] v;                 /* buffer for reallocation */

    if (this.size < this.xs.length)
      this.resize(this.size);   /* shrink the coord. vectors if poss. */
    this.dists = new double[this.size][this.size];
    if (!calc) return;          /* create the distance matrix */
    for (i = this.size; --i >= 0; ) {
      this.dists[i][i] = 0;     /* set diagonal elements to zero */
      for (k = i; --k >= 0; ) { /* traverse the off-diagonal elements */
        dx = this.xs[i] -this.xs[k];
        dy = this.ys[i] -this.ys[k];
        this.dists[i][k] = this.dists[k][i] = Math.sqrt(dx*dx +dy*dy);
      }                         /* compute pairwise vertex distances */
    }                           /* (Euclidian distances, symmetric) */
    this.euclid = this.sym = true;
  }  /* makeDists() */

  /*------------------------------------------------------------------*/
  /** Check whether the traveling salesman problem is symmetric.
   *  @return whether the traveling salesman problem is symmetric
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public boolean isSymmetric ()
  { return this.sym; }

  /*------------------------------------------------------------------*/
  /** Get the distance between two vertices.
   *  @param  i  the index of the first  vertex
   *  @param  j  the index of the second vertex
   *  @return the distance between the vertices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public double getDist (int i, int j)
  { return this.dists[i][j]; }

  /*------------------------------------------------------------------*/
  /** Set the (symmetric) distance between two vertices.
   *  @param  i     the index of the first  vertex
   *  @param  j     the index of the second vertex
   *  @param  dist  the distance between the vertices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void setDist  (int i, int j, double dist)
  { this.dists[i][j] = this.dists[j][i] = dist; this.euclid = false; }

  /*------------------------------------------------------------------*/
  /** Set the asymmetric distance between two vertices.
   *  @param  i     the index of the first  vertex
   *  @param  j     the index of the second vertex
   *  @param  dist  the distance between the vertices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void    setDistAsym (int i, int j, double dist)
  { this.dists[i][j] = dist; this.sym = false;  this.euclid = false; }

  /*------------------------------------------------------------------*/
  /** Get the current tour.
   *  @return the current tour as an array of vertex indices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public int[] getTour ()
  { return this.tour; }

  /*------------------------------------------------------------------*/
  /** Set the current tour.
   *  @param  tour  the tour to set as an array of vertex indices
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public void  setTour (int[] tour)
  {                             /* --- set a (best) tour */
    if (this.tour == null)      /* create a tour if necessary */
      this.tour = new int[this.size];
    System.arraycopy(tour, 0, this.tour, 0, this.size);
  }  /* setTour() */            /* copy the given tour */

  /*------------------------------------------------------------------*/
  /** Describe the traveling salesman problem (TSP) as a string.
   *  @return a string description of the traveling salesman problem
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public String toString ()
  {                             /* --- create a string description */
    int           i, k;         /* loop variables */
    StringBuilder s;            /* created string description */

    s = new StringBuilder("TSP = {\n");
    s.append("  vertices = {"); /* there are always vertices */
    s.append("\n    (" +this.xs[0] +", " +this.ys[0] +")");
    for (i = 1; i < this.size; i++)
      s.append(",\n    (" +this.xs[i] +", " +this.ys[i] +")");
    s.append("\n  };\n");       /* list the vertices */
    if (!this.euclid) {         /* if the distances are not Euclidean */
      s.append("  distances = {");
      for (i = 0; i < this.size; i++) {
        if (i > 0) s.append(",");
        s.append("\n    { " +this.dists[i][0]);
        for (k = 1; k < this.size; k++)
          s.append(", " +this.dists[i][k]);
        s.append(" }");         /* list the pairwise distances */
      }                         /* (which are possibly asymmetric, */
      s.append("\n  };\n");     /* so that the full distance matrix */
    }                           /* may be needed) */
    if (this.tour != null) {    /* if a (best) tour is known */
      s.append("  tour = {\n    " +this.tour[0]);
      for (i = 1; i < this.size; i++)
        s.append(", " +this.tour[i]);
      s.append("\n  };\n");     /* list the vertices of the tour */
    }
    s.append("};\n");           /* terminate the description */
    return s.toString();        /* return the created string */
  }  /* toString() */

  /*------------------------------------------------------------------*/
  /** Read a traveling salesman problem from a file.
   *  @param  s  the scanner to read from
   *  @throws IOException if a read error occurred
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (Scanner s) throws IOException
  {                             /* --- read a traveling salesman p. */
    this();                     /* initialize the variables */
    int    i, k;                /* loop variables */
    double x, y;                /* buffers for coordinates */

    /* --- read vertices --- */
    s.getID("TSP");             /* check for keyword 'TSP' */
    s.getChar('=');             /* check for a '=' */
    s.getChar('{');             /* check for a '{' */
    s.getID("vertices");        /* check for keyword 'vertices' */
    s.getChar('=');             /* check for a '=' */
    s.getChar('{');             /* check for a '{' */
    if (s.nextToken() == '}')   /* if the list is empty, abort */
      throw new IOException("no vertices in TSP");
    s.pushBack();               /* push back already read token */
    do {                        /* vertex read loop */
      s.getChar('(');           /* check for a '(' */
      s.getNumber();            /* get the x-coordinate */
      x = Double.parseDouble(s.value);
      s.getChar(',');           /* check for a ',' */
      s.getNumber();            /* get the y-coordinate */
      y = Double.parseDouble(s.value);
      s.getChar(')');           /* check for a ')' */
      if (this.size >= this.xs.length)
        this.resize(-1);        /* resize the coord. vectors if nec. */
      this.xs[this.size  ] = x; /* store the coordinates */
      this.ys[this.size++] = y; /* of the new vertex */
    } while (s.nextToken() == ',');
    s.pushBack();               /* push back last token */
    s.getChar('}');             /* check for a '}' */
    s.getChar(';');             /* check for a ';' */
    if (this.size < this.xs.length)
      this.resize(this.size);   /* shrink coord. vectors if possible */

    /* --- read distances --- */
    if ((s.nextToken() != Scanner.T_ID)
    ||  !s.value.equals("distances")) {
      this.makeDists(true);     /* calculate distances if necessary */
      s.pushBack(); }           /* (symmetric, Euclidean distances) */
    else {                      /* if distances are given */
      s.getChar('=');           /* check for a '=' */
      s.getChar('{');           /* check for a '{' */
      this.dists = new double[this.size][this.size];
      for (i = 0; i < this.size; i++) {
        if (i > 0) s.getChar(',');
        s.getChar('{');         /* check for a '{' */
        for (k = 0; k < this.size; k++) {
          if (k > 0) s.getChar(',');
          s.getNumber();        /* check for a number */
          this.dists[i][k] = Double.parseDouble(s.value);
        }                       /* store the distance */
        s.getChar('}');         /* check for a '}' */
      }
      this.euclid = false;      /* distances were read */
      s.getChar('}');           /* check for a '}' */
      s.getChar(';');           /* check for a ';' */
      this.sym = true;          /* default: symmetric distances */
      for (i = this.size; (--i >= 0) && this.sym; ) {
        for (k = i; --k >= 0; )
          if (this.dists[i][k] != this.dists[k][i]) {
            this.sym = false; break; }
      }                         /* check for symmetric distances */
    }                           /* and set flag accordingly */

    /* --- read tour --- */
    if ((s.nextToken() != Scanner.T_ID)
    ||  !s.value.equals("tour"))
      s.pushBack();             /* check whether a tour is known */
    else {                      /* if there is a tour */
      s.getChar('=');           /* check for a '=' */
      s.getChar('{');           /* check for a '{' */
      for (i = 0; i < this.size; i++) {
        if (i > 0) s.getChar(',');
        s.getNumber();          /* check for a number */
        this.tour[i] = Integer.parseInt(s.value);
      }                         /* store the vertex index */
      s.getChar('}');           /* check for a '}' */
      s.getChar(';');           /* check for a ';' */
    }

    s.getChar('}');             /* check for a '}' */
    s.getChar(';');             /* check for a ';' */
  }  /* TSP() */

  /*------------------------------------------------------------------*/
  /** Read a traveling salesman problem from a string.
   *  @param  desc  the string description to read from
   *  @throws IOException if a read error occurred
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (String desc)    throws IOException
  { this(new Scanner(desc)); }

  /*------------------------------------------------------------------*/
  /** Read a traveling salesman problem from an input stream.
   *  @param  in  the input stream to read from
   *  @throws IOException if a read error occurred
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (InputStream in) throws IOException
  { this(new Scanner(in)); }

  /*------------------------------------------------------------------*/
  /** Read a traveling salesman problem from a reader.
   *  @param  reader  the reader to read from
   *  @throws IOException if a read error occurred
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public TSP (Reader reader)  throws IOException
  { this(new Scanner(reader)); }

  /*------------------------------------------------------------------*/
  /** Main program.
   *  @param  args  the command line arguments
   *  @since  2005.11.19 (Christian Borgelt) */
  /*------------------------------------------------------------------*/

  public static void main (String[] args)
  {                             /* --- main function for testing */
    int    size;                /* number of vertices */
    double scale = 1.0;         /* scaling factor */
    long   seed;                /* seed value for random numbers */
    TSP    tsp;                 /* created random TSP */

    if (args.length <= 0) {     /* if no file name is given */
      System.err.println("usage: TSP <vertcnt> [<scale>] [<seed>]");
      return;                   /* print a usage message */
    }                           /* and abort the program */
    seed = System.currentTimeMillis();
    size = Integer.parseInt(args[0]);
    if (args.length > 1) scale = Double.parseDouble(args[1]);
    if (args.length > 2) seed  = Integer.parseInt(args[2]);
    tsp = new TSP(size, new Random(seed));
    tsp.transform(scale,0,0);   /* create a TSP and scale it */
    System.out.print(tsp);      /* print the created TSP */
  }  /* main() */

}  /* class TSP */
